#!/usr/bin/env python
# encoding: utf-8

from path import Path
from glob import glob

def get_projects(demux_dir, unaligned_dir='Unaligned'):
    """TODO: Docstring for get_projects.

    Args:
        demux_dir (TODO): TODO

    Returns: TODO

    """

    projects = []
 
    project_dirs = glob(Path(demux_dir).joinpath(unaligned_dir, '*'))
    for project_dir in project_dirs:
        project = Path(project_dir).normpath().basename()
        if project.startswith('Project_'):
            project = project.split('_', 1)[1]
            projects.append(project)

    return projects

def gather_flowcell(demux_dir):
    """TODO: Docstring for gather_flowcell.

    Args:
        demux_dir (str): path to demux dir

    Returns: TODO

    """

    rs = {} # result set

    # get the flowcell name
    full_flowcell_name = Path(demux_dir).normpath().basename().split('_')[-1]
    rs['name'] = full_flowcell_name[1:]

    # get the flowcell position: A|B
    rs['pos'] = full_flowcell_name[0]

    return rs



# -*- coding: utf-8 -*-
from datetime import datetime
import logging

import click
import json
import yaml

from .export import export_run
from .models import Analysis
from .mip import process_all

log = logging.getLogger(__name__)


@click.group()
def analysis():
    """Interact with the post alignment part of the database."""
    pass


@analysis.command()
@click.option('-f', '--force', is_flag=True)
@click.argument('sampleinfo_file', type=click.File('r'))
@click.argument('metrics_file', type=click.File('r'))
@click.pass_context
def add(context, force, sampleinfo_file, metrics_file):
    """Load data from analysis output."""
    sampleinfo = yaml.load(sampleinfo_file)
    analysis_id = '-'.join([sampleinfo['owner'], sampleinfo['family']])
    if not force and not test_analysis(sampleinfo):
        log.warn("analysis can't be loaded, use '--force'")
        context.abort()
    else:
        old_analysis = Analysis.query.filter_by(analysis_id=analysis_id).first()
        if old_analysis:
            if force:
                log.info("removing old analysis")
                old_analysis.delete()
            else:
                log.debug("analysis already added: %s", analysis_id)
                context.abort()

    metrics = yaml.load(metrics_file)
    log.debug("parsing analysis: %s", analysis_id)
    new_analysis = process_all(analysis_id, sampleinfo, metrics)
    log.info("adding analysis: %s", new_analysis.analysis_id)
    context.obj['manager'].add_commit(new_analysis)


@analysis.command()
@click.argument('case_id')
@click.pass_context
def delete(context, case_id):
    """Delete an existing analysis and samples in the database."""
    analysis_obj = Analysis.query.filter_by(analysis_id=case_id).first()
    if analysis_obj is None:
        log.error('analysis not found in database')
        context.abort()
    analysis_obj.delete()
    context.obj['manager'].commit()
    log.info("removed analysis: %s", analysis_obj.id)


def test_analysis(sampleinfo):
    """Test if it's a supported version of MIP."""
    status = sampleinfo['analysisrunstatus']
    if status != 'finished':
        return False

    if not sampleinfo['mip_version'].startswith('v4.'):
        return False

    return True


@analysis.command()
@click.option('-c', '--condense', is_flag=True)
@click.argument('existing_data', type=click.File('r'), default='-')
@click.pass_context
def export(context, condense, existing_data):
    """Export interesting data about a case."""
    existing = yaml.load(existing_data)
    new_data = export_run(context.obj['manager'], existing)
    if condense:
        raw_dump = json.dumps(new_data, default=json_serial)
    else:
        raw_dump = yaml.safe_dump(new_data, default_flow_style=False,
                                  allow_unicode=True)
    click.echo(raw_dump)


def fillin_data(existing_data, case_data, samples_data):
    """Fill in data about a case with cgstats data."""
    existing_data.update(case_data)

    # fill in sample data
    for sample_data in existing_data['samples']:
        new_data = samples_data[sample_data['id']]
        sample_data.update(new_data)

    return existing_data


def json_serial(obj):
    """JSON serializer for objects not serializable by default json code"""
    if isinstance(obj, datetime):
        serial = obj.isoformat()
        return serial
    raise TypeError('Type not serializable')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import socket
from datetime import datetime
from path import Path

from cgstats.db.parse import gather_supportparams, gather_datasource, get_basemask

def test_gather_supportparams(rapid_run_dir):
    demuxdir = Path(rapid_run_dir)
    assert gather_supportparams(demuxdir, 'Unaligned') == {
        'idstring': 'bcl2fastq-1.8.4',
        'program': '/usr/local/bin/configureBclToFastq.pl',
        'command': {
            '--sample-sheet': 'tests/fixtures/150114_D00134_0168_AHB07NADXX/Data/Intensities/BaseCalls/SampleSheet.csv',
            '--use-bases-mask': 'Y101,I6n,Y101'
        },
        'system': {
            'OS': 'linux',
            'PID': '47668',
            'PERL_VERSION': 'v5.10.1',
            'PERL_EXECUTABLE': '/usr/bin/perl'
        },
        'document_path': str(demuxdir.joinpath('Unaligned')),
        'commandline': '/usr/local/bin/configureBclToFastq.pl --sample-sheet tests/fixtures/150114_D00134_0168_AHB07NADXX/Data/Intensities/BaseCalls/SampleSheet.csv --use-bases-mask Y101,I6n,Y101 --fastq-cluster-count 0 --input-dir tests/fixtures/150114_D00134_0168_AHB07NADXX/Data/Intensities/BaseCalls --output-dir DEMUX/150114_D00134_0168_AHB07NADXX/Unaligned',
        'sampleconfig_path': str(demuxdir.joinpath('SampleSheet.csv')),
        'sampleconfig': 'FCID,Lane,SampleID,SampleRef,Index,Description,Control,Recipe,Operator,SampleProject\n'
                        'HB07NADXX,1,SIB911A1_sureselect4,hg19,TGACCA,959191,N,R1,NN,959191\n'
                        'HB07NADXX,1,SIB911A2_sureselect5,hg19,ACAGTG,959191,N,R1,NN,959191\n'
                        'HB07NADXX,1,SIB910A3_sureselect6,hg19,GCCAAT,454557,N,R1,NN,454557\n'
                        'HB07NADXX,1,SIB914A2_sureselect2,hg19,CGATGT,504910,N,R1,NN,504910\n'
                        'HB07NADXX,1,SIB914A11_sureselect11,hg19,GGCTAC,504910,N,R1,NN,504910\n'
                        'HB07NADXX,1,SIB914A12_sureselect12,hg19,CTTGTA,504910,N,R1,NN,504910\n'
                        'HB07NADXX,1,SIB914A15_sureselect15,hg19,GAAACC,504910,N,R1,NN,504910\n'
                        'HB07NADXX,2,SIB911A1_sureselect4,hg19,TGACCA,959191,N,R1,NN,959191\n'
                        'HB07NADXX,2,SIB911A2_sureselect5,hg19,ACAGTG,959191,N,R1,NN,959191\n'
                        'HB07NADXX,2,SIB910A3_sureselect6,hg19,GCCAAT,454557,N,R1,NN,454557\n'
                        'HB07NADXX,2,SIB914A2_sureselect2,hg19,CGATGT,504910,N,R1,NN,504910\n'
                        'HB07NADXX,2,SIB914A11_sureselect11,hg19,GGCTAC,504910,N,R1,NN,504910\n'
                        'HB07NADXX,2,SIB914A12_sureselect12,hg19,CTTGTA,504910,N,R1,NN,504910\n'
                        'HB07NADXX,2,SIB914A15_sureselect15,hg19,GAAACC,504910,N,R1,NN,504910',
        'time': datetime.fromtimestamp(demuxdir.joinpath('Unaligned', 'support.txt').getmtime()) # can't really test this
        }


def test_gather_datasource(rapid_run_dir):
    assert gather_datasource(Path(rapid_run_dir)) == {
        'rundate': u'150114',
        'machine': u'D00134',
        'runname': '150114_D00134_0168_AHB07NADXX',
        'servername': socket.gethostname()
    }

def test_get_basemask(rapid_run_dir):
    demuxdir = Path(rapid_run_dir)
    supportparams = gather_supportparams(demuxdir, 'Unaligned')

    assert get_basemask(supportparams) == 'Y101,I6n,Y101'



# -*- coding: utf-8 -*-
import logging

from .models import Analysis

log = logging.getLogger(__name__)


def fillin_data(existing_data, case_data, samples_data):
    """Fill in data about a case with cgstats data."""
    existing_data.update(case_data)

    # filter out samples that are not in the analysis
    relevant_samples = []
    for sample_data in existing_data['samples']:
        if sample_data['id'] in samples_data:
            relevant_samples.append(sample_data)
        else:
            log.warn("skipping sample: %s", sample_data['id'])
    existing_data['samples'] = relevant_samples

    # fill in sample data
    for sample_data in existing_data['samples']:
        new_data = samples_data[sample_data['id']]
        sample_data.update(new_data)

    return existing_data


def export_run(cgstats_db, existing_data):
    """Fill in export information based on LIMS data."""
    case_id = existing_data['case_id']
    analysis_obj = Analysis.query.filter_by(analysis_id=case_id).first()
    if analysis_obj is None:
        log.error('analysis not found in database')
        return None
    case_data = {
        'pipeline': analysis_obj.pipeline,
        'pipeline_version': analysis_obj.pipeline_version,
        'analysis_date': analysis_obj.analyzed_at,
    }
    samples_data = {}
    for sample_obj in analysis_obj.samples:
        sample_data = {
            'sequencing_type': sample_obj.sequencing_type,
            'sex_predicted': sample_obj.sex_predicted,
            'read_pairs': sample_obj.read_pairs,
            'mapped': sample_obj.mapped_percent,
            'duplicates': sample_obj.duplicates_percent,
            'target_coverage': sample_obj.coverage_target,
            'target_completeness': sample_obj.completeness_target_10,
        }
        samples_data[sample_obj.sample_id] = sample_data

    new_data = fillin_data(existing_data, case_data, samples_data)
    return new_data



#!/usr/bin/env python
# encoding: utf-8

from __future__ import print_function, division

import xml.etree.cElementTree as et
import glob
import re
import os
import logging

from path import Path

from demux.utils import Samplesheet


log = logging.getLogger(__name__)


def xpathsum(tree, xpath):
    """Sums all numbers found at these xpath nodes

    Args:
        tree (an elementTree): parsed XML as an elementTree
        xpath (str): an xpath the XML nodes

    Returns (int): the sum of all nodes

    """
    numbers = tree.findall(xpath)
    return sum(( int(number.text) for number in numbers ))

def get_barcode_summary(tree, project, sample, barcode):
    """Calculates following statistics from the DemultiplexingStats file
    * BarcodeCount
    * PerfectBarcodeCount
    * OneMismatchBarcodeCount

    Args:
        tree (an elementTree): parsed XML as an elementTree

    Returns: TODO

    """
    barcodes = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/BarcodeCount".format(project=project, sample=sample, barcode=barcode))
    perfect_barcodes = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/PerfectBarcodeCount".format(project=project, sample=sample, barcode=barcode))
    one_mismatch_barcodes = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/OneMismatchBarcodeCount".format(project=project, sample=sample, barcode=barcode))

    return {
        'barcodes': barcodes,
        'perfect_barcodes': perfect_barcodes,
        'one_mismatch_barcodes': one_mismatch_barcodes,
    }

def get_sample_summary( tree, project, sample, barcode):
    """Calculates following statistics from from the ConversionStats file, for a sample
    * pf clusters
    * pf yield
    * pf Q30
    * raw Q30
    * pf Q Score

    Args:
        tree (an elementTree): parsed XML as an elementTree
        project (str): A project name
        sample (str): A sample name. In our case, this is the same as the project
        barcode (str): An index identifying a sample

    Returns (dict): with following keys: pf_clusters, pf_yield, pf_q30, pf_read1_yield, pf_read2_yield, pf_read1_q30, pf_read2_q30, pf_qscore_sum, pf_qscore

    """
    raw_clusters = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/Tile/Raw/ClusterCount".format(project=project, sample=sample, barcode=barcode))
    pf_clusters = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/Tile/Pf/ClusterCount".format(project=project, sample=sample, barcode=barcode))

    pf_yield = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/Tile/Pf/Read/Yield".format(project=project, sample=sample, barcode=barcode))
    log.debug("\tpf_yield={}".format(pf_yield))
    pf_read1_yield = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/Tile/Pf/Read[@number='1']/Yield".format(project=project, sample=sample, barcode=barcode))
    pf_read2_yield = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/Tile/Pf/Read[@number='2']/Yield".format(project=project, sample=sample, barcode=barcode))
    raw_yield = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/Tile/Raw/Read/Yield".format(project=project, sample=sample, barcode=barcode))
    #pf_clusters_pc = pf_yield / raw_yield

    pf_q30 = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/Tile/Pf/Read/YieldQ30".format(project=project, sample=sample, barcode=barcode))
    #raw_q30 = xpathsum(tree, "./Stats/Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/Tile/Raw/Read/YieldQ30".format(project=project, sample=sample, barcode=barcode))
    pf_read1_q30 = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/Tile/Pf/Read[@number='1']/YieldQ30".format(project=project, sample=sample, barcode=barcode))
    pf_read2_q30 = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/Tile/Pf/Read[@number='2']/YieldQ30".format(project=project, sample=sample, barcode=barcode))
    #pf_q30_bases_pc = pf_q30 / pf_yield

    pf_qscore_sum = xpathsum(tree, "Flowcell/Project[@name='{project}']/Sample[@name='{sample}']/Barcode[@name='{barcode}']/Lane/Tile/Pf/Read/QualityScoreSum".format(project=project, sample=sample, barcode=barcode))
    pf_qscore = pf_qscore_sum / pf_yield if pf_yield != 0 else 0

    return {
        #'pf_q30_bases_pc': pf_q30_bases_pc,
        #'raw_q30': raw_q30,
        #'pf_clusters_pc': pf_clusters_pc,
        'raw_clusters': raw_clusters,
        'raw_yield': raw_yield,
        'pf_clusters': pf_clusters,
        'pf_yield': pf_yield,
        'pf_read1_yield': pf_read1_yield,
        'pf_read2_yield': pf_read2_yield,
        'pf_q30': pf_q30,
        'pf_read1_q30': pf_read1_q30,
        'pf_read2_q30': pf_read2_q30,
        'pf_qscore_sum': pf_qscore_sum,
        'pf_qscore': pf_qscore
    }


def calc_undetermined( demux_dir):
    sizes = {}
    all_files = glob.glob(demux_dir + '/l*/Project*/Sample*/*fastq.gz')
    for f in all_files:
        sample_name = re.search(r'Sample_(.*)/', f).group(1)
        if sample_name not in sizes:
            sizes[ sample_name ] = { 'size_of': 0, 'u_size_of': 0 }
        sizes[ sample_name ]['size_of'] += os.path.getsize(f)

    und_files = glob.glob(demux_dir + '/l*/Project*/Sample*/Undet*fastq.gz')
    for f in und_files:
        sample_name = re.search(r'Sample_(.*)/', f).group(1)
        sizes[ sample_name ]['u_size_of'] += os.path.getsize(f)

    proc_undetermined = {}
    for sample_name, size in sizes.items():
        proc_undetermined[ sample_name ] = float(size['u_size_of']) / size['size_of'] * 100

    return proc_undetermined

def get_raw_clusters_lane(total_sample_summary):
    """TODO: Docstring for get_raw_clusters_lane.

    Args:
        total_sample_summary (TODO): TODO

    Returns: TODO

    """
    raw_clusters_lane = dict(zip(total_sample_summary.keys(), [ 0 for t in range(len(total_sample_summary.keys())) ])) # lane: raw_clusters
    for lane, sample_summary in total_sample_summary.items():
        for sample, summary in sample_summary.items():
            raw_clusters_lane[ lane ] += summary['raw_clusters']

    return raw_clusters_lane

def parse_samples(demux_dir):
    """Takes a DEMUX dir and calculates statistics for a run on sample level.

    Args:
        demux_dir (str): the DEMUX dir

    """
    log.debug("Parsing sample stats ...")

    samplesheet = Samplesheet(Path(demux_dir).joinpath('SampleSheet.csv'))
    samples = list(set(samplesheet.samples()))
    lanes = list(set(samplesheet.column('lane')))

    # create a { 1: {}, 2: {}, ... } structure
    summaries = {lane_key: {} for lane_key in lanes}

    # preload the stats files
    et_stats_files = dict(zip(lanes, [ [] for t in range(len(lanes))]))
    et_index_files = dict(zip(lanes, [ [] for t in range(len(lanes))]))
    for lane in lanes:

        stats_files = glob.glob('{}/l{}t??/Stats/ConversionStats.xml'.format(demux_dir, lane))
        index_files = glob.glob('{}/l{}t??/Stats/DemultiplexingStats.xml'.format(demux_dir, lane))

        if len(stats_files) == 0:
            exit("No stats file found for sample {}".format(sample))

        if len(index_files) == 0:
            exit("No index stats file found for sample {}".format(sample))

        for f in stats_files:
            et_stats_files[lane].append(et.parse(f))

        for f in index_files:
            et_index_files[lane].append(et.parse(f))

    # get all the stats numbers
    #import ipdb; ipdb.set_trace()
    for sample in samples:
        log.debug("Getting stats for '{}'...".format(sample))
        for line in samplesheet.lines_per_column('sample_id', sample):
            lane = line['lane']
            log.debug("...for lane {}".format(lane))
            if sample not in summaries[lane]: summaries[lane][sample] = [] # init some more

            for tree in et_stats_files[lane]:
                summaries[lane][sample].append(get_sample_summary(tree, line['project'], line['sample_name'], line['index']))

            for tree in et_index_files[ line['lane'] ]:
                summaries[lane][sample].append(get_barcode_summary(tree, line['project'], line['sample_name'], line['index']))

    # sum the numbers over a lane
    # create a { 1: {}, 2: {}, ... } structure
    total_sample_summary = dict(zip(lanes, [ {} for t in range(len(lanes))]))
    for line in samplesheet.lines():
        total_sample_summary[ line['lane'] ][ line['sample_id'] ] = {
            'raw_clusters': 0,
            'raw_yield': 0,
            'pf_clusters': 0,
            'pf_yield': 0,
            'pf_read1_yield': 0,
            'pf_read2_yield': 0,
            'pf_q30': 0,
            'pf_read1_q30': 0,
            'pf_read2_q30': 0,
            'pf_qscore_sum': 0,
            'pf_qscore': 0,
            'flowcell': line['fcid'],
            'samplename': line['sample_id'],
            'lane': line['lane'],
            'barcodes': 0,
            'perfect_barcodes': 0,
            'one_mismatch_barcodes': 0,
        }

    for lane, sample_summary in summaries.items():
        for sample, summary in sample_summary.items():
            for summary_quart in summary:
                for key, stat in summary_quart.items():
                    total_sample_summary[lane][sample][ key ] += stat

    raw_clusters_lane = get_raw_clusters_lane(total_sample_summary)

    rs= dict(zip(lanes, [ {} for t in range(len(lanes))]))
    for lane, sample_summary in total_sample_summary.items():
        for sample, summary in sample_summary.items():
            rs[ lane ][ summary['samplename'] ] = {
                'sample_name':     summary['samplename'],
                'flowcell':        summary['flowcell'],
                'lane':            lane,
                'raw_clusters_pc': round(summary['raw_clusters'] / raw_clusters_lane[lane] * 100, 2),
                'pf_clusters':     summary['pf_clusters'],
                'pf_yield_pc':     round(summary['pf_yield'] / summary['raw_yield'] * 100, 2),
                'pf_yield':        summary['pf_yield'],
                'pf_Q30':          round(summary['pf_q30'] / summary['pf_yield'] * 100, 2),
                'pf_read1_q30':    round(summary['pf_read1_q30'] / summary['pf_read1_yield'] * 100, 2),
                'pf_read2_q30':    round(summary['pf_read2_q30'] / summary['pf_read2_yield'] * 100, 2),
                'pf_qscore':       round(summary['pf_qscore_sum'] / summary['pf_yield'], 2),
                'undetermined_pc': (summary['pf_clusters'] - summary['barcodes']) / summary['pf_clusters'] * 100,
                'barcodes':         summary['barcodes'],
                'perfect_barcodes': summary['perfect_barcodes'],
                'one_mismatch_barcodes': summary['one_mismatch_barcodes'],
            }

    return rs

def parse( demux_dir):
    """Takes a DEMUX dir and calculates statistics for the run.

    Args:
        demux_dir (str): the DEMUX dir

    """

    log.debug("Parsing on lane level ...")

    samplesheet = Samplesheet(Path(demux_dir).joinpath('SampleSheet.csv'))
    lanes = list(set(samplesheet.column('lane')))

    # create a { 1: [], 2: [], ... } structure
    summaries = dict(zip(lanes, [ [] for t in range(len(lanes))])) # init ;)

    # preload the stats files
    et_stats_files = dict(zip(lanes, [ [] for t in range(len(lanes))]))
    et_index_files = dict(zip(lanes, [ [] for t in range(len(lanes))]))
    for lane in lanes:
        stats_files = glob.glob('{}/l{}t??/Stats/ConversionStats.xml'.format(demux_dir, lane))
        index_files = glob.glob('{}/l{}t??/Stats/DemultiplexingStats.xml'.format(demux_dir, lane))

        if len(stats_files) == 0:
            exit("No stats file found for lane {}".format(lane))

        if len(index_files) == 0:
            exit("No index stats file found for lane {}".format(lane))

        for f in stats_files:
            et_stats_files[lane].append(et.parse(f))

        for f in index_files:
            et_index_files[lane].append(et.parse(f))

    # get all the stats numbers
    for lane in lanes:

        # only parse this on lane level
        for tree in et_stats_files[lane]:
            summaries[lane].append(get_sample_summary(tree, 'all', 'all', 'all'))

        # we need barcode stats on sample level
        for line in samplesheet.lines_per_column('lane', lane):
            for tree in et_index_files[ lane ]:
                summaries[lane].append(get_barcode_summary(tree, line['project'], line['sample_name'], line['index']))

    # sum the numbers over a lane
    # create a { 1: {'raw_clusters': 0, ... } } structure
    total_lane_summary = {}
    for line in samplesheet.lines():
        total_lane_summary[ line['lane'] ] = {
            'raw_clusters': 0,
            'raw_yield': 0,
            'pf_clusters': 0,
            'pf_yield': 0,
            'pf_read1_yield': 0,
            'pf_read2_yield': 0,
            'pf_q30': 0,
            'pf_read1_q30': 0,
            'pf_read2_q30': 0,
            'pf_qscore_sum': 0,
            'pf_qscore': 0,
            'flowcell': line['fcid'],
            'samplename': line['sample_id'],
            'barcodes': 0,
            'perfect_barcodes': 0,
            'one_mismatch_barcodes': 0,
        }

    for lane, summary in summaries.items():
       for summary_quart in summary:
            for key, stat in summary_quart.items():
                total_lane_summary[lane][ key ] += stat

    rs = {} # generate a dict: raw sample name is key, value is a dict of stats
    for lane, summary in total_lane_summary.items():
        rs[ lane ] = {
            'sample_name':     summary['samplename'],
            'flowcell':        summary['flowcell'],
            'lane':            lane,
            'raw_clusters_pc': 100, # we still only have one sample/lane ;)
            'pf_clusters':     summary['pf_clusters'],
            'pf_yield_pc':     round(summary['pf_yield'] / summary['raw_yield'] * 100, 2),
            'pf_yield':        summary['pf_yield'],
            'pf_Q30':          round(summary['pf_q30'] / summary['pf_yield'] * 100, 2),
            'pf_read1_q30':    round(summary['pf_read1_q30'] / summary['pf_read1_yield'] * 100, 2),
            'pf_read2_q30':    round(summary['pf_read2_q30'] / summary['pf_read2_yield'] * 100, 2),
            'pf_qscore':       round(summary['pf_qscore_sum'] / summary['pf_yield'], 2),
            'undetermined_pc': (summary['pf_clusters'] - summary['barcodes']) / summary['pf_clusters'] * 100,
            'barcodes':         summary['barcodes'],
            'perfect_barcodes': summary['perfect_barcodes'],
            'one_mismatch_barcodes': summary['one_mismatch_barcodes'],
        }

    return rs

__ALL__ = [ 'parse', 'parse_samples' ]



# -*- coding: utf-8 -*-
import codecs
import logging
import os
import pkg_resources

import click
import yaml

from cgstats import __title__, __version__
from cgstats.db import api

log = logging.getLogger(__name__)


def init_log(logger, filename=None, loglevel=None):
    """Initializes the log file in the proper format.

    Args:
        filename (Optional[path]): default is no logging to file
        loglevel (Optional[str]): determine the level of log output
    """
    template = "[%(asctime)s] %(levelname)-8s: %(name)-25s: %(message)s"
    formatter = logging.Formatter(template)

    if loglevel:
        logger.setLevel(getattr(logging, loglevel))

    # We will always print warnings and higher to stderr
    console = logging.StreamHandler()
    console.setLevel('WARNING')
    console.setFormatter(formatter)

    if filename:
        file_handler = logging.FileHandler(filename, encoding='utf-8')
        if loglevel:
            file_handler.setLevel(getattr(logging, loglevel))
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)
    # If no logfile is provided we print all log messages that the user has
    # defined to stderr
    else:
        if loglevel:
            console.setLevel(getattr(logging, loglevel))

    logger.addHandler(console)


class EntryPointsCLI(click.MultiCommand):
    """Add subcommands dynamically to a CLI via entry points."""

    def _iter_commands(self):
        """Iterate over all subcommands as defined by the entry point."""
        subcommands_ep = "{}.subcommands.1".format(__title__)
        return {entry_point.name: entry_point for entry_point in
                pkg_resources.iter_entry_points(subcommands_ep)}

    def list_commands(self, ctx):
        """List the available commands."""
        commands = self._iter_commands()
        return commands.keys()

    def get_command(self, ctx, name):
        """Load one of the available commands."""
        commands = self._iter_commands()
        if name not in commands:
            click.echo("no such command: {}".format(name))
            ctx.abort()
        return commands[name].load()


@click.group(cls=EntryPointsCLI)
@click.option('-c', '--config', default='~/.cgstats.yaml',
              type=click.Path(), help='path to config file')
@click.option('-d', '--database', help='path/URI of the SQL database')
@click.option('-l', '--log-level', default='INFO', help='level to log at')
@click.option('-r', '--reset', is_flag=True,
              help='reset database from scratch')
@click.option('--log-file', type=click.Path(), help='write logs to a file')
@click.version_option(__version__, prog_name=__title__)
@click.pass_context
def root(context, config, database, reset, log_level, log_file):
    """Interact with command line interface."""
    init_log(logging.getLogger(), loglevel=log_level, filename=log_file)
    log.debug("{}: version {}".format(__title__, __version__))
    # read in config file if it exists
    if os.path.exists(config):
        with codecs.open(config) as conf_handle:
            context.obj = yaml.load(conf_handle)
    else:
        context.obj = {}

    if database:
        context.obj['database'] = database
    if context.obj.get('database'):
        context.obj['manager'] = api.connect(context.obj['database'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from cgstats.utils.utils import get_projects, gather_flowcell

def test_get_projects(rapid_run_dir, miseq_run_dir):
    assert sorted(get_projects(rapid_run_dir, 'Unaligned')) == sorted(['454557', '504910', '959191'])
    assert sorted(get_projects(miseq_run_dir, 'Unaligned-Y301I8I8Y301')) == ['Uppsala_WGS']

def test_gather_flowcell_rapid(rapid_run_dir):
    assert gather_flowcell(rapid_run_dir) == {'pos': 'A', 'name': 'HB07NADXX'}

def test_gather_flowcell_x(x_run_dir):
    assert gather_flowcell(x_run_dir) == {'pos': 'A', 'name': 'HC7H2ALXX'}



# -*- coding: utf-8 -*-
"""Parse QC metrics file from MIP."""
from __future__ import division
import logging

from .models import Analysis, AnalysisSample

log = logging.getLogger(__name__)


def parse_versions(qcmetrics):
    """Parse our version from family section of data."""
    versions = {}
    programs = ['samtools', 'gatk', 'freebayes', 'genmod', 'manta']
    for program in programs:
        program_data = qcmetrics['program'].get(program)
        if program_data:
            versions[program] = str(program_data['version'])
        else:
            log.warn("missing version info for: %s", program)
    return versions


def build_sample(sample_id, sample_data, seq_type):
    """Parse out relevant information from sample data."""
    data = dict(sample_id=sample_id, sequencing_type=seq_type)
    for key, value in sample_data.items():
        if '_lanes_' in key:
            # alignment
            hs_metrics = value['calculatehsmetrics']['header']['data']
            mult_metrics = value['collectmultiplemetrics']['header']['pair']
            data['strand_balance'] = mult_metrics['STRAND_BALANCE']
            data['sex_predicted'] = value['chanjo_sexcheck']['gender']

            # coverage
            data['coverage_target'] = hs_metrics['MEAN_TARGET_COVERAGE']
            data['completeness_target_10'] = hs_metrics['PCT_TARGET_BASES_10X']
            data['completeness_target_20'] = hs_metrics['PCT_TARGET_BASES_20X']
            data['completeness_target_50'] = hs_metrics['PCT_TARGET_BASES_50X']
            data['completeness_target_100'] = hs_metrics['PCT_TARGET_BASES_100X']

            # variants
            comp_overlap = (value['variantevalall']['comp_overlap_header']
                                 ['comp_overlap_data_all'])
            variant_sum = (value['variantevalall']['variant_summary_header']
                                ['variant_summary_data_all'])
            variant_count = (value['variantevalall']['count_variants_header']
                                  ['count_variants_data_all'])
            data['variants'] = comp_overlap['nEvalVariants']
            data['indels'] = variant_sum['nIndels']
            data['snps'] = variant_sum['nSNPs']
            data['titv_ratio'] = variant_sum['TiTvRatio']
            data['novel_sites'] = comp_overlap['novelSites']
            data['concordant_rate'] = comp_overlap['concordantRate'] / 100
            data['hethom_ratio'] = variant_count['hetHomRatio']

            data['duplicates_percent'] = (value['markduplicates']
                                               ['fraction_duplicates'])

    data['reads_total'] = sample_data['reads']
    data['mapped_percent'] = sample_data['reads_mapped_rate']

    new_sample = AnalysisSample(**data)
    return new_sample


def build_analysis(analysis_id, sampleinfo):
    """Build analysis model from QC sample info file."""
    rank_model = sampleinfo['program']['rankvariant']['rank_model']['version']
    data = dict(
        pipeline='mip',
        analysis_id=analysis_id,
        pipeline_version=sampleinfo['mip_version'],
        analyzed_at=sampleinfo['analysis_date'],
        program_versions=dict(rank_model=rank_model),
    )
    return Analysis(**data)


def process_all(analysis_id, sampleinfo, qcmetrics):
    """Process all data."""
    new_analysis = build_analysis(analysis_id, sampleinfo)
    # parse out program versions
    versions = parse_versions(qcmetrics)
    versions['rank_model'] = new_analysis.program_versions['rank_model']
    new_analysis.program_versions = versions

    for sample_id, sample_data in qcmetrics['sample'].items():
        log.info("adding sample: %s", sample_id)
        seq_type = sampleinfo['analysis_type'][sample_id]
        new_sample = build_sample(sample_id, sample_data, seq_type)
        capture_kit = sampleinfo['sample'][sample_id].get('capture_kit')
        new_sample.capture_kit = capture_kit
        new_analysis.samples.append(new_sample)

    return new_analysis



# -*- coding: utf-8 -*-
import logging
import click
from path import Path
from glob import glob

from .models import Flowcell, Version, Sample, Demux, Unaligned
from . import api
from . import xparse
from . import parse
from ..utils import xstats

log = logging.getLogger(__name__)


@click.command()
@click.option('-l', '--limit', default=10)
@click.argument('flowcell_id', required=False)
@click.pass_context
def show(context, limit, flowcell_id):
    """Show flowcells as JSON objects."""
    if flowcell_id:
        flowcells = Flowcell.query.filter_by(flowcellname=flowcell_id)
    else:
        flowcells = Flowcell.query.limit(limit)
    for flowcell in flowcells:
        click.echo(flowcell.to_json(pretty=True))


@click.command()
@click.option('-e', '--expected', type=int, help='expected number of reads')
@click.option('-d', '--date', is_flag=True, help='show date of last sequencing')
@click.argument('sample_id')
@click.pass_context
def sample(context, expected, date, sample_id):
    """Report how many reads a sample has been sequenced."""
    # fetch sample from the database
    query = api.get_sample(sample_id)
    num_samples = query.count()
    if num_samples == 0:
        click.echo("sample not found in database")
        context.abort()
    elif num_samples > 1:
        samples_str = ", ".join(sample.samplename for sample in query)
        click.echo("conflicting samples: {}".format(samples_str))
        context.abort()
    else:
        sample_obj = query.one()

    # fetch number of reads
    reads = sum(unaligned.readcounts for unaligned in sample_obj.unaligned)
    if expected and reads < expected:
        log.error("not enough reads! ({})".format(reads))
        context.abort()

    if date:
        sorted_dates = sorted(unaligned.demux.datasource.rundate for unaligned
                              in sample_obj.unaligned)
        click.echo(sorted_dates[-1])
    else:
        click.echo(reads)


@click.command()
@click.option('-l', '--limit', default=20, help='limit number of flowcells')
@click.option('-o', '--offset', default=0, help='skip initial flowcells')
@click.option('-s', '--sample', help='look up flowcells based on sample')
@click.option('-v', '--verbose', is_flag=True, help='show more information')
@click.pass_context
def flowcells(context, limit, offset, sample, verbose):
    """Get information about flowcells."""
    query = api.flowcells(sample=sample)
    for flowcell in query.offset(offset).limit(limit):
        if verbose:
            click.echo(flowcell.to_json(pretty=True))
        else:
            click.echo(flowcell.flowcellname)


@click.command()
@click.option('-l', '--limit', default=20, help='limit number of flowcells')
@click.option('-o', '--offset', default=0, help='skip initial flowcells')
@click.option('-f', '--flowcell', help='flowcell name')
@click.pass_context
def samples(context, limit, offset, flowcell):
    """List samples in the database."""
    query = api.samples(flowcell_name=flowcell)
    if flowcell is None:
        query = query.offset(offset).limit(limit)
    for sample in query:
        click.echo(sample.lims_id)


@click.command()
@click.argument('demux_dir')
@click.pass_context
def lanestats(context, demux_dir):
    """List lane centric stats for X"""
    lane_stats = sorted([ line for _, line in xstats.parse(demux_dir).items()], key=lambda line: int(line['lane']))

    click.echo("Flowcell\tLane\tReads\tYieldMB\tQ30\tQ30_read1\tQ30_read2\tMeanQScore\tUndetermined")

    for line in lane_stats:
        click.echo('\t'.join(str(s) for s in [
            line['flowcell'],
            line['lane'],
            line['pf_clusters'],
            int(round(line['pf_yield'] / 1000000, 0)),
            line['pf_Q30'],
            line['pf_read1_q30'],
            line['pf_read2_q30'],
            line['pf_qscore'],
            round(line['undetermined_pc'], 2)
        ]))


@click.command()
@click.argument('flowcell')
@click.option('-p', '--project', help='project name')
@click.pass_context
def select(context, flowcell, project):
    """List sample centric stats"""
    query = api.select(flowcell, project)

    click.echo("sample\tFlowcell\tLanes\treadcounts/lane\tsum_readcounts\tyieldMB/lane\tsum_yield\t%Q30\tMeanQscore")
    for line in query:
            click.echo('\t'.join( str(s) for s in [line.samplename, line.flowcellname, line.lanes, line.reads, line.readsum, line.yld, line.yieldsum, line.q30, line.meanq] ))


@click.command()
@click.argument('demux_dir')
@click.option('-m', '--machine', type=click.Choice(['X', '2500']), help='machine type')
@click.option('-u', '--unaligned', help='the unaligned dir name')
@click.option('-a', '--all-unaligned', is_flag=True, help='add all Unaligned* dirs')
@click.pass_context
def add(context, machine, demux_dir, unaligned, all_unaligned):
    """Add a FC to cgstats."""

    #if not Version.check(config['clinstats']['name'], config['clinstats']['version']):
    #    logger.error('Wrong database!')
    #    exit(1) # change to exception

    manager = context.obj['manager']

    if machine == 'X':
        xparse.add(manager, demux_dir)
    if machine == '2500':
        if all_unaligned:
            unaligned_dirs = glob(Path(demux_dir).joinpath('Unaligned*'))
            for unaligned in unaligned_dirs:
                log.info('Adding {}.'.format(unaligned))
                parse.add(manager, demux_dir, unaligned)
        else:
            log.info('Adding {}.'.format(unaligned))
            parse.add(manager, demux_dir, unaligned)

@click.command()
@click.option('-f', '--flowcell', help='flowcell to remove')
@click.option('-b', '--basemask', help='demux with given basemask to remove')
@click.option('-s', '--sample', help='sample to remove')
@click.pass_context
def delete(context, flowcell, basemask, sample):

    if flowcell:
        rs = Flowcell.query.filter_by(flowcellname=flowcell)

    if basemask:
        if not flowcell:
            click.echo('Missing flowcell parameter', err=True)
            context.abort()
        rs = Demux.query.filter_by(basemask=basemask)

    if sample:
        rs = Sample.query.filter_by(limsid=sample)
        if flowcell:
            rs = rs.join(Unaligned).join(Demux).join(Flowcell).filter_by(flowcellname=flowcell)

    rs = rs.all()
    manager = context.obj['manager']
    if click.confirm("Are you sure you want to delete {}?".format(rs)):
        manager.delete(rs)
        manager.commit()



#!/usr/bin/env python
# encoding: utf-8

from __future__ import print_function
import sys
from pprint import pprint
from .utils import xstats
import pstats
import cProfile


def main(argv):
    
    #print(Supportparams.exists('/home/clinical/DEMUX//150703_D00134_0206_AH5HGFBCXX/Unaligned4/support.txt')) # 515
    #print(Datasource.exists('/home/clinical/DEMUX//150703_D00134_0206_AH5HGFBCXX/Unaligned4/Basecall_Stats_H5HGFBCXX/Demultiplex_Stats.htm')) #515
    #print(Flowcell.exists('H5HGFBCXX')) # 512
    #print(Demux.exists(512, 'Y101,I8,I8,Y101')) # 474
    #print(Project.exists('240540')) #552
    #print(Sample.exists('ADM1136A1_XTA08', 'CAGCGTTA')) #6651
    #print(Unaligned.exists(18, 487, 1)) #13902

    #pprint(xstats.parse_samples('/mnt/hds/proj/bioinfo/DEMUX/160308_ST-E00269_0055_AHK5VKCCXX/'))
    pprint(xstats.parse_samples('/mnt/hds/proj/bioinfo/DEMUX/161125_ST-E00269_0150_AH37GVALXX'))
    pprint(xstats.parse_samples('/mnt/hds/proj/bioinfo/DEMUX/161125_ST-E00269_0150_AH37GVALXX'))
    #pprint(xstats.parse_samples('/mnt/hds/proj/bioinfo/DEMUX/160308_ST-E00269_0055_AHK5VKCCXX/'))

    #print(Backup.exists('151117_D00410_0187_AHWYGMADXX'))
    #print(Backup.exists('141212_D00134_0166_AHB058ADXX'))
    #print(Backup.exists('131219_D00134_0057_BH829YADXX'))
    #print(Backup.exists('141028_D00410_0109_AHAH5WADXX'))
    #print(Backup.exists('131219_D00134_0057_BH829YADXX', 'tape005_006'))
    #print(Backup.exists('131219_D00134_0057_BH829YADXX', 'tape007_005'))

if __name__ == '__main__':
    outfile = 'xpathstats_bs_fqp_parse'
    cProfile.run('xstats.parse("/mnt/hds/proj/bioinfo/DEMUX/161125_ST-E00269_0150_AH37GVALXX")', outfile)
    p = pstats.Stats(outfile)
    p.sort_stats('cumtime').print_stats(10)

    outfile = 'xpathstats_bs_fqp_samples_parse'
    cProfile.run('xstats.parse_samples("/mnt/hds/proj/bioinfo/DEMUX/161125_ST-E00269_0150_AH37GVALXX")', outfile)
    p = pstats.Stats(outfile)
    p.sort_stats('cumtime').print_stats(10)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from path import Path

from cgstats.utils.utils import gather_flowcell
from cgstats.db import parse
from cgstats.db import xparse
from cgstats.db.models import Flowcell, Sample, Demux, Unaligned, Datasource, Supportparams, Project
from cgstats.db import api

wgs_sample_count = 8
wes_sample_count = 7
wes8_sample_count = 10
wes9_sample_count = 8
wgs_lane_count = 8
wes_lane_count = 2

def test_db_add(sql_manager, rapid_run_dir, x_run_dir):
    """ Add a rapid flowcell. We know the rowcounts for all tables.
    Add an X flowcell to add some more data.

    * do we get the right amount of counts for all tables?
    * check the presence of a randomly chosen sample.
    * add the flowcells again. Duplicates?
    """

    def _count_all_tables():
        # ok, do we have the right counts ...
        samples = Sample.query.all()
        projects = Project.query.all()
        flowcells = Flowcell.query.all()
        demuxes = Demux.query.all()
        unaligneds = Unaligned.query.all()
        datasources = Datasource.query.all()
        supportparams = Supportparams.query.all()
        assert len(samples) == wgs_sample_count + wes_sample_count
        assert len(projects) == 4
        assert len(flowcells) == 2
        assert len(demuxes) == 2

        # unaligned will have one row for each sample and for each lane it was run on
        assert len(unaligneds) == wgs_sample_count + wes_sample_count * wes_lane_count
        assert len(datasources) == 2
        assert len(supportparams) == 2

    unaligned = 'Unaligned'
    parse.add(sql_manager, rapid_run_dir, unaligned)
    xparse.add(sql_manager, x_run_dir) # adds 8 wgs samples

    # let's just quickly check if the added information is correct
    samples = Sample.query.filter_by(limsid='SIB914A11').all()
    assert len(samples) == 1
    sample = samples.pop()
    assert sample.limsid == 'SIB914A11'
    assert sample.samplename == 'SIB914A11_sureselect11'

    _count_all_tables()

    # let's add it again and check if we have duplicates
    parse.add(sql_manager, rapid_run_dir, unaligned)
    xparse.add(sql_manager, x_run_dir)
    samples = Sample.query.filter_by(limsid='SIB914A11').all()
    assert len(samples) == 1

    _count_all_tables()

def test_db_api(sql_manager, rapid_run_dir, x_run_dir):

    # add some stuff
    test_db_add(sql_manager, rapid_run_dir, x_run_dir)

    # test
    flowcells = api.flowcells(sample='SIB914A11').all()
    assert len(flowcells) == 1
    assert flowcells.pop().flowcellname == gather_flowcell(rapid_run_dir)['name']


def test_select(sql_manager, rapid_run_dir, x_run_dir):
    """ Tests out one query that touches a lot of the tables.
    Do we get the expected result? """

    flowcell = gather_flowcell(rapid_run_dir)['name']
    project = '504910'
    unaligned = 'Unaligned'
    xparse.add(sql_manager, x_run_dir)
    parse.add(sql_manager, rapid_run_dir, unaligned)
    selection = api.select(flowcell, project).all()

    assert selection == [
        ('SIB914A11_sureselect11', 'HB07NADXX', '1,2', '38088672,38269896', 76358568, '3847,3865', 7712, '93.71,93.70', '36.27,36.27'),
        ('SIB914A12_sureselect12', 'HB07NADXX', '1,2', '48201748,48191852', 96393600, '4868,4867', 9735, '94.39,94.40', '36.49,36.49'),
        ('SIB914A15_sureselect15', 'HB07NADXX', '1,2', '57947620,57997530', 115945150, '5853,5858', 11711, '94.32,94.33', '36.46,36.46'),
        ('SIB914A2_sureselect2',   'HB07NADXX', '1,2', '32032000,32016648', 64048648, '3235,3234', 6469, '94.11,94.12', '36.40,36.40')
    ]

def test_db_delete_sample(sql_manager, rapid_run_dir, x_run_dir):
    """ Add a rapid and a X flowcell. Delete a rapid sample.

    * Expected: one Sample less. Two Unaligned rows less.
    """
    unaligned = 'Unaligned'

    xparse.add(sql_manager, x_run_dir) # adds 8 wgs samples
    parse.add(sql_manager, rapid_run_dir, unaligned) # adds 7 WES samples all run on two lanes

    # make sure the sample is there to start with
    sample = Sample.query.filter(Sample.limsid == 'SIB914A11').all()
    assert len(sample) == 1

    # let's try to delete a sample
    sql_manager.delete(sample)
    sql_manager.commit()

    # ok, do we have the right counts ...
    sample = Sample.query.filter(Sample.limsid == 'SIB914A11').all()
    samples = Sample.query.all()
    flowcells = Flowcell.query.all()
    demuxes = Demux.query.all()
    unaligneds = Unaligned.query.all()
    datasources = Datasource.query.all()
    assert len(sample) == 0

    # one sample is removed
    assert len(samples) == wes_sample_count + wgs_sample_count - 1

    # one sample removed from the rapid run, means 2 unaligned rows removed
    assert len(unaligneds) == wes_sample_count * wes_lane_count + wgs_sample_count - 2

    assert len(flowcells) == 2
    assert len(demuxes) == 2
    assert len(datasources) == 2

def test_db_delete_demux(sql_manager, rapid_run_dir, x_run_dir, mixed_rapid_run_dir):
    """ Add a rapid and an X flowcell. Delete the demux row of the rapid.
    This should remove all rows from all tables related to the rapid flowcell.
    """
    unaligned = 'Unaligned'

    xparse.add(sql_manager, x_run_dir)
    parse.add(sql_manager, rapid_run_dir, unaligned)

    # is the demux there?
    demux = Demux.query.filter(Demux.basemask == 'Y101,I6n,Y101').all()
    assert len(demux) == 1

    # let's try to delete a demux
    sql_manager.delete(demux)
    sql_manager.commit()

    # ok, do we have the counts ...
    demux = Demux.query.all()
    unaligneds = Unaligned.query.join(Sample).all()
    samples = Sample.query.all()
    flowcells = Flowcell.query.all()
    datasources = Datasource.query.all()
    supportparams = Supportparams.query.all()
    assert len(demux) == 1
    assert len(unaligneds) == wgs_sample_count
    assert len(samples) == wgs_sample_count
    assert len(flowcells) == 2 # does not automatically delete FC when it's an orphan
    assert len(datasources) == 1
    assert len(supportparams) == 1

    # now the most important test: add a variable length index rapid FC and remove one demux
    parse.add(sql_manager, mixed_rapid_run_dir, 'Unaligned8', 'SampleSheet8.csv')
    parse.add(sql_manager, mixed_rapid_run_dir, 'Unaligned9', 'SampleSheet9.csv')

    # are the demuxes there?
    demux8 = Demux.query.filter(Demux.basemask == 'Y126,I8,I8,Y126').all()
    demux9 = Demux.query.filter(Demux.basemask == 'Y126,I8,n8,Y126').all()
    assert len(demux8) == 1
    assert len(demux9) == 1

    # let's remove one demux
    sql_manager.delete(demux8)
    sql_manager.commit()

    # ok, do we have the counts ...
    demux = Demux.query.all()
    unaligneds = Unaligned.query.join(Sample).all()
    samples = Sample.query.all()
    flowcells = Flowcell.query.all()
    datasources = Datasource.query.all()
    supportparams = Supportparams.query.all()
    assert len(demux) == 2
    assert len(unaligneds) == wgs_sample_count + wes9_sample_count * wes_lane_count
    assert len(samples) == wgs_sample_count + wes9_sample_count
    assert len(flowcells) == 3
    assert len(datasources) == 2
    assert len(supportparams) == 2


def test_db_delete_flowcell(sql_manager, rapid_run_dir, x_run_dir, x_pooled_run_dir):
    """ Add a rapid and a X flowclel. Delete the X flowcell.
    This should remove all rows from all tables related to the rapid flowcell."""
    unaligned = 'Unaligned'
    flowcell_name = gather_flowcell(x_run_dir)['name']

    # let's try to delete a flowcell
    xparse.add(sql_manager, x_run_dir)
    parse.add(sql_manager, rapid_run_dir, unaligned)

    # is the flowcell there?
    flowcell = Flowcell.query.filter(Flowcell.flowcellname == flowcell_name).all()
    assert len(flowcell) == 1

    # let's try to delete a demux
    sql_manager.delete(flowcell)
    sql_manager.commit()

    # ok, do we have the counts ...
    flowcells = Flowcell.query.all()
    unaligneds = Unaligned.query.join(Sample).all()
    samples = Sample.query.all()
    demux = Demux.query.all()
    datasources = Datasource.query.all()
    supportparams = Supportparams.query.all()
    assert len(demux) == 1
    assert len(unaligneds) == wes_sample_count * wes_lane_count
    assert len(samples) == wes_sample_count
    assert len(flowcells) == 1
    assert len(datasources) == 1
    assert len(supportparams) == 1



#!/usr/bin/env python
# encoding: utf-8

from __future__ import print_function, division

import sys
import os
import glob
import time
import re
import socket
import ast
from datetime import datetime

from sqlalchemy import func
from path import Path

from demux.utils import HiSeq2500Samplesheet
from cgstats.utils import stats as hiseqstats
from cgstats.utils.utils import get_projects, gather_flowcell
from cgstats.db.models import Supportparams, Version, Datasource, Flowcell, Demux, Project, Sample, Unaligned

def gather_supportparams(demuxdir, unaligneddir):
    """
    """
    support_file = demuxdir.joinpath(unaligneddir, 'support.txt')

    system_str = ""
    command_str = ""
    idstring = ""
    program = ""

    lines = (line.strip() for line in open(support_file))
    for line in lines:
        if line.startswith('$_System'):
            line = line.replace('$_System = ', '')
            system_str += line + '\n'
            continue

        if system_str:
            if line.startswith("};"):
                system_str += '}'
                break
            else:
                system_str += line + '\n'

    system = ast.literal_eval(system_str)

    for line in lines:
        if line.startswith('$_ID-string'):
            idstring = line.replace("$_ID-string = '","")
            idstring = idstring.replace("';","")
        if line.startswith('$_Program'):
            program = line.replace("$_Program = '","")
            program = program.replace("';","")
            break

    for line in lines:
        if line.startswith('$_Command-line'):
            line = line.replace('$_Command-line = ', '')
            command_str += line + '\n'
            continue

        if command_str:
            if line.startswith("];"):
                command_str += ']'
                break
            else:
                command_str += line + '\n'

    c = ast.literal_eval(command_str)

    command = {
        '--sample-sheet': c[ c.index('--sample-sheet') + 1],
        '--use-bases-mask': c[ c.index('--use-bases-mask') + 1],
    }

    samplesheet_path = str(demuxdir.joinpath('SampleSheet.csv'))
    samplesheet = HiSeq2500Samplesheet(samplesheet_path)

    return {
        'system': system,
        'command': command,
        'program': program,
        'idstring': idstring,
        'document_path': str(demuxdir.joinpath(unaligneddir)),
        'commandline': ' '.join([ program, ' '.join(c)]),
        'sampleconfig_path': samplesheet_path,
        'sampleconfig': samplesheet.raw(),
        'time': datetime.fromtimestamp(support_file.getmtime())
    }

def gather_datasource(demuxdir):
    rs = {} # resultset

    runname = demuxdir.normpath().basename()
    name_parts = runname.split("_")

    rs['runname'] = str(runname)
    rs['rundate'] = name_parts[0]
    rs['machine'] = name_parts[1]
    rs['servername'] = socket.gethostname()

    return rs

def get_basemask(supportparams):
    return supportparams['command']['--use-bases-mask']

def add(manager, demux_dir, unaligned_dir, samplesheet_name = 'SampleSheet.csv'):
    """TODO: Docstring for add.
    Returns: TODO

    """

    demux_dir = Path(demux_dir)
    demux_stats = glob.glob(demux_dir.joinpath(unaligned_dir, 'Basecall_Stats_*', 'Demultiplex_Stats.htm'))[0]

    samplesheet_path = demux_dir.joinpath(samplesheet_name)
    samplesheet = HiSeq2500Samplesheet(samplesheet_path)

    stats = hiseqstats.parse(demux_stats)

    supportparams_id = Supportparams.exists(demux_dir.joinpath(unaligned_dir))
    new_supportparams = gather_supportparams(demux_dir, unaligned_dir)
    if not supportparams_id:
        supportparams = Supportparams()
        supportparams.document_path = new_supportparams['document_path']
        supportparams.idstring = new_supportparams['idstring']
        supportparams.program = new_supportparams['program']
        supportparams.commandline = new_supportparams['commandline']
        supportparams.sampleconfig_path = new_supportparams['sampleconfig_path']
        supportparams.sampleconfig = new_supportparams['sampleconfig']
        supportparams.time = new_supportparams['time']

        manager.add(supportparams)
        manager.flush()
        supportparams_id = supportparams.supportparams_id

    datasource_id = Datasource.exists(str(demux_stats))
    if not datasource_id:
        new_datasource = gather_datasource(demux_dir)
        datasource = Datasource()
        datasource.runname = new_datasource['runname']
        datasource.rundate = new_datasource['rundate']
        datasource.machine = new_datasource['machine']
        datasource.server = new_datasource['servername']
        datasource.document_path = str(demux_stats)
        datasource.document_type = 'html'
        datasource.time = datetime.fromtimestamp(Path(demux_stats).getmtime())
        datasource.supportparams_id = supportparams_id

        manager.add(datasource)
        manager.flush()
        datasource_id = datasource.datasource_id

    flowcell_namepos = gather_flowcell(demux_dir)
    flowcell_name = flowcell_namepos['name']
    flowcell_pos  = flowcell_namepos['pos']
    flowcell_id   = Flowcell.exists(flowcell_name)
    if not flowcell_id:
        flowcell = Flowcell()
        flowcell.flowcellname = flowcell_name
        flowcell.flowcell_pos = flowcell_pos
        flowcell.hiseqtype = 'hiseqga'
        flowcell.time = func.now()

        manager.add(flowcell)
        manager.flush()
        flowcell_id = flowcell.flowcell_id

    basemask = get_basemask(new_supportparams)
    demux_id = Demux.exists(flowcell_id, basemask)
    if not demux_id:
        demux = Demux()
        demux.flowcell_id = flowcell_id
        demux.datasource_id = datasource_id
        demux.basemask = basemask
        demux.time = func.now()

        manager.add(demux)
        manager.flush()
        demux_id = demux.demux_id

    project_id_of = {} # project name: project id
    for project_name in get_projects(demux_dir, unaligned_dir):
        project_id = Project.exists(project_name)
        if not project_id:
            p = Project()
            p.projectname = project_name
            p.time = func.now()

            manager.add(p)
            manager.flush()
            project_id = p.project_id

        project_id_of[ project_name ] = project_id

    for line in samplesheet.lines():
        sample_id = Sample.exists(line['sample_id'], line['index'])
        if not sample_id:
            s = Sample()
            s.project_id = project_id_of[ line['project'] ]
            s.samplename = line['sample_id']
            s.limsid = line['sample_id'].split('_')[0]
            s.barcode = line['index']
            s.time = func.now()

            manager.add(s)
            manager.flush()
            sample_id = s.sample_id

        if not Unaligned.exists(sample_id, demux_id, line['lane']):
            u = Unaligned()
            u.sample_id = sample_id
            u.demux_id = demux_id
            u.lane = line['lane']
            stats_sample = stats[line['lane']][line['sample_id']]

            u.yield_mb = int(stats_sample['yield_mb'])
            u.passed_filter_pct = stats_sample['pf_pc']
            u.readcounts = stats_sample['readcounts']
            u.raw_clusters_per_lane_pct = stats_sample['raw_clusters_pc']
            u.perfect_indexreads_pct = stats_sample['perfect_barcodes_pc']
            u.q30_bases_pct = stats_sample['q30_bases_pc']
            u.mean_quality_score = stats_sample['mean_quality_score']
            u.time = func.now()

            manager.add(u)

    manager.flush()
    manager.commit()

    return True



from cgstats.utils import stats

def test_missing_parse():
    """After demuxing this run, stats failed on empty pf_pc of P012. This test is to make sure we get a 0 now"""

    rs = stats.parse('tests/fixtures/170616_M03284_0062_000000000-B58CV/Unaligned-Y301I8I8Y301/Basecall_Stats_000000000-B58CV/Demultiplex_Stats.htm')

    assert rs['1']['P012']['pf_pc'] == 0

def test_parse():
    assert stats.parse('tests/fixtures/150114_D00134_0168_AHB07NADXX/Unaligned/Basecall_Stats_HB07NADXX/Demultiplex_Stats.htm') == {
        '1': {
            'SIB910A3_sureselect6': {'barcode': 'GCCAAT',
                                    'lane': '1',
                                    'mean_quality_score': 36.45,
                                    'perfect_barcodes_pc': 100.00,
                                    'pf_pc': 100.00,
                                    'project_id': '454557',
                                    'q30_bases_pc': 94.33,
                                    'raw_clusters_pc': 13.33,
                                    'readcounts': 43225752,
                                    'sample_name': 'SIB910A3_sureselect6',
                                    'yield_mb': 4366},
            'SIB911A1_sureselect4': {'barcode': 'TGACCA',
                                    'lane': '1',
                                    'mean_quality_score': 36.51,
                                    'perfect_barcodes_pc': 100.00,
                                    'pf_pc': 100.00,
                                    'project_id': '959191',
                                    'q30_bases_pc': 94.48,
                                    'raw_clusters_pc': 15.96,
                                    'readcounts': 51778316,
                                    'sample_name': 'SIB911A1_sureselect4',
                                    'yield_mb': 5230},
            'SIB911A2_sureselect5': {'barcode': 'ACAGTG',
                                    'lane': '1',
                                    'mean_quality_score': 36.46,
                                    'perfect_barcodes_pc': 100.00,
                                    'pf_pc': 100.00,
                                    'project_id': '959191',
                                    'q30_bases_pc': 94.31,
                                    'raw_clusters_pc': 12.20,
                                    'readcounts': 39563494,
                                    'sample_name': 'SIB911A2_sureselect5',
                                    'yield_mb': 3996},
            'SIB914A11_sureselect11': {'barcode': 'GGCTAC',
                                      'lane': '1',
                                      'mean_quality_score': 36.27,
                                      'perfect_barcodes_pc': 100.00,
                                      'pf_pc': 100.00,
                                      'project_id': '504910',
                                      'q30_bases_pc': 93.71,
                                      'raw_clusters_pc': 11.74,
                                      'readcounts': 38088672,
                                      'sample_name': 'SIB914A11_sureselect11',
                                      'yield_mb': 3847},
            'SIB914A12_sureselect12': {'barcode': 'CTTGTA',
                                      'lane': '1',
                                      'mean_quality_score': 36.49,
                                      'perfect_barcodes_pc': 100.00,
                                      'pf_pc': 100.00,
                                      'project_id': '504910',
                                      'q30_bases_pc': 94.39,
                                      'raw_clusters_pc': 14.86,
                                      'readcounts': 48201748,
                                      'sample_name': 'SIB914A12_sureselect12',
                                      'yield_mb': 4868},
            'SIB914A15_sureselect15': {'barcode': 'GAAACC',
                                      'lane': '1',
                                      'mean_quality_score': 36.46,
                                      'perfect_barcodes_pc': 100.00,
                                      'pf_pc': 100.00,
                                      'project_id': '504910',
                                      'q30_bases_pc': 94.32,
                                      'raw_clusters_pc': 17.87,
                                      'readcounts': 57947620,
                                      'sample_name': 'SIB914A15_sureselect15',
                                      'yield_mb': 5853},
            'SIB914A2_sureselect2': {'barcode': 'CGATGT',
                                    'lane': '1',
                                    'mean_quality_score': 36.40,
                                    'perfect_barcodes_pc': 100.00,
                                    'pf_pc': 100.00,
                                    'project_id': '504910',
                                    'q30_bases_pc': 94.11,
                                    'raw_clusters_pc': 9.88,
                                    'readcounts': 32032000,
                                    'sample_name': 'SIB914A2_sureselect2',
                                    'yield_mb': 3235},
            'lane1': {'barcode': 'Undetermined',
                     'lane': '1',
                     'mean_quality_score': 35.46,
                     'perfect_barcodes_pc': 0.00,
                     'pf_pc': 100.00,
                     'project_id': 'Undetermined_indices',
                     'q30_bases_pc': 91.15,
                     'raw_clusters_pc': 4.17,
                     'readcounts': 13520174,
                     'sample_name': 'lane1',
                     'yield_mb': 1366}},
        '2': {
            'SIB910A3_sureselect6': {'barcode': 'GCCAAT',
                                    'lane': '2',
                                    'mean_quality_score': 36.45,
                                    'perfect_barcodes_pc': 100.00,
                                    'pf_pc': 100.00,
                                    'project_id': '454557',
                                    'q30_bases_pc': 94.32,
                                    'raw_clusters_pc': 13.31,
                                    'readcounts': 43286568,
                                    'sample_name': 'SIB910A3_sureselect6',
                                    'yield_mb': 4372},
            'SIB911A1_sureselect4': {'barcode': 'TGACCA',
                                    'lane': '2',
                                    'mean_quality_score': 36.51,
                                    'perfect_barcodes_pc': 100.00,
                                    'pf_pc': 100.00,
                                    'project_id': '959191',
                                    'q30_bases_pc': 94.48,
                                    'raw_clusters_pc': 15.93,
                                    'readcounts': 51823804,
                                    'sample_name': 'SIB911A1_sureselect4',
                                    'yield_mb': 5234},
            'SIB911A2_sureselect5': {'barcode': 'ACAGTG',
                                    'lane': '2',
                                    'mean_quality_score': 36.46,
                                    'perfect_barcodes_pc': 100.00,
                                    'pf_pc': 100.00,
                                    'project_id': '959191',
                                    'q30_bases_pc': 94.33,
                                    'raw_clusters_pc': 12.18,
                                    'readcounts': 39619404,
                                    'sample_name': 'SIB911A2_sureselect5',
                                    'yield_mb': 4002},
            'SIB914A11_sureselect11': {'barcode': 'GGCTAC',
                                      'lane': '2',
                                      'mean_quality_score': 36.27,
                                      'perfect_barcodes_pc': 100.00,
                                      'pf_pc': 100.00,
                                      'project_id': '504910',
                                      'q30_bases_pc': 93.70,
                                      'raw_clusters_pc': 11.76,
                                      'readcounts': 38269896,
                                      'sample_name': 'SIB914A11_sureselect11',
                                      'yield_mb': 3865},
            'SIB914A12_sureselect12': {'barcode': 'CTTGTA',
                                      'lane': '2',
                                      'mean_quality_score': 36.49,
                                      'perfect_barcodes_pc': 100.00,
                                      'pf_pc': 100.00,
                                      'project_id': '504910',
                                      'q30_bases_pc': 94.40,
                                      'raw_clusters_pc': 14.81,
                                      'readcounts': 48191852,
                                      'sample_name': 'SIB914A12_sureselect12',
                                      'yield_mb': 4867},
            'SIB914A15_sureselect15': {'barcode': 'GAAACC',
                                      'lane': '2',
                                      'mean_quality_score': 36.46,
                                      'perfect_barcodes_pc': 100.00,
                                      'pf_pc': 100.00,
                                      'project_id': '504910',
                                      'q30_bases_pc': 94.33,
                                      'raw_clusters_pc': 17.83,
                                      'readcounts': 57997530,
                                      'sample_name': 'SIB914A15_sureselect15',
                                      'yield_mb': 5858},
            'SIB914A2_sureselect2': {'barcode': 'CGATGT',
                                    'lane': '2',
                                    'mean_quality_score': 36.40,
                                    'perfect_barcodes_pc': 100.00,
                                    'pf_pc': 100.00,
                                    'project_id': '504910',
                                    'q30_bases_pc': 94.12,
                                    'raw_clusters_pc': 9.84,
                                    'readcounts': 32016648,
                                    'sample_name': 'SIB914A2_sureselect2',
                                    'yield_mb': 3234},
            'lane2': {'barcode': 'Undetermined',
                     'lane': '2',
                     'mean_quality_score': 35.46,
                     'perfect_barcodes_pc': 0.00,
                     'pf_pc': 100.00,
                     'project_id': 'Undetermined_indices',
                     'q30_bases_pc': 91.17,
                     'raw_clusters_pc': 4.34,
                     'readcounts': 14103704,
                     'sample_name': 'lane2',
                     'yield_mb': 1424}}}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import io
import os
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
import sys

# This is a plug-in for setuptools that will invoke py.test
# when you run python setup.py test
class PyTest(TestCommand):

    """Set up the py.test test runner."""

    def finalize_options(self):
        """Set options for the command line."""
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        """Execute the test runner command."""
        # Import here, because outside the required eggs aren't loaded yet
        import pytest
        sys.exit(pytest.main(self.test_args))

# Get the long description from the relevant file
here = os.path.abspath(os.path.dirname(__file__))
with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

with open('requirements.txt') as f:
    required = [ line for line in f.read().splitlines() if not line.startswith('-e') ]

setup(
    name='cgstats',

    # Versions should comply with PEP440. For a discussion on
    # single-sourcing the version across setup.py and the project code,
    # see http://packaging.python.org/en/latest/tutorial.html#version
    version='1.1.3',
    description='Models and access to cgstats',
    long_description=long_description,
    # What does your project relate to? Separate with spaces.
    author='Kenny Billiau',
    author_email='kenny.billiau@scilifelab.se',
    license='MIT',

    packages=find_packages(exclude=('tests*', 'docs', 'examples')),

    # If there are data files included in your packages that need to be
    # installed, specify them here.
    include_package_data=True,
    zip_safe=False,

    install_requires=required,
    cmdclass=dict(test=PyTest),

    # See: http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are:
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Science/Research',
        'Topic :: Software Development',
        'Topic :: Scientific/Engineering :: Bio-Informatics',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',

        'Environment :: Console',
    ],
    platforms='any',
    entry_points={
        'console_scripts': [
            'cgstats = cgstats.cli:root',
        ],
        'cgstats.subcommands.1': [
            'init = cgstats.initialize:init',
            'show = cgstats.db.cli:show',
            'analysis = cgstats.analysis.cli:analysis',
            'sample = cgstats.db.cli:sample',
            'flowcells = cgstats.db.cli:flowcells',
            'samples = cgstats.db.cli:samples',
            'add = cgstats.db.cli:add',
            'delete = cgstats.db.cli:delete',
            'select = cgstats.db.cli:select',
            'lanestats = cgstats.db.cli:lanestats',
        ],
        'cgstats.models.1': [
            'core = cgstats.db.models',
            'analysis = cgstats.analysis.models',
        ]
    },
)



# -*- coding: utf-8 -*-
import logging

import click

log = logging.getLogger(__name__)


@click.command()
@click.option('-r', '--reset', is_flag=True)
@click.pass_context
def init(context, reset):
    """Setup the clinstats database."""
    manager = context.obj['manager']
    if reset:
        manager.drop_all()
    manager.create_all()
